﻿-------------------------------------------------------------------------------
-- Addon - Runes - Init
-------------------------------------------------------------------------------
-- Version
local MAJOR = "Avion2_Runes"
local MINOR = tonumber(("10"):match("%d+")) or 0

-- Dependencies
if not Avion2Core 				then error(MAJOR .. " requires Avion2.") end
if not LibStub("AceEvent-3.0")	then error(MAJOR .. " requires AceEvent-3.0.") end

-- Create
Avion2_Runes 		= Avion2Core:NewModule(MAJOR, "AceEvent-3.0")
Avion2_Runes.major 	= MAJOR
Avion2_Runes.minor 	= MINOR


-------------------------------------------------------------------------------
-- Addon - Runes - Variables
-------------------------------------------------------------------------------
-- Translation
local L 	= LibStub("AceLocale-3.0"):GetLocale(MAJOR)
local coreL	= LibStub("AceLocale-3.0"):GetLocale(Avion2Core.major)

-- Save settings for new trigger
local newSettings = {
	runes	= {
		["RUNETYPE_BLOOD"] 		= 0,
		["RUNETYPE_CHROMATIC"] 	= 0,
		["RUNETYPE_FROST"] 		= 0,
		["RUNETYPE_DEATH"] 		= 0,
	},
	rune		= 0,
	runeType 	= 0,
	power 		= 0,
}

-- Local functions
local RunesRelay, PowerRelay, RunesUpdate, PowerUpdate, PowerRunesUpdate


-------------------------------------------------------------------------------
-- Addon - Runes - Register with core
-------------------------------------------------------------------------------
-- Register SavedVariables
function Avion2_Runes:RegisterModuleDB()
	-- Register own saved variables DB
	self.db = Avion2Core.db:RegisterNamespace(Avion2_Runes.major, {
		profile = {
			combinations = {
				data				= {},
				onlyHighestRunes 	= false,
				onlyHighestPower 	= false,
			},
			
			power 		= {
				data			= {},
				onlyOneActive 	= false,
			},
			
			runes		= {
				data			= {},
				onlyOneActive 	= false,
			},
			
			lastMinor 					= MINOR,
		},
	})
	
	-- Last Runes
	self.lastRunes = {
		["RUNETYPE_BLOOD"] 		= 0,
		["RUNETYPE_CHROMATIC"] 	= 0,
		["RUNETYPE_FROST"] 		= 0,
		["RUNETYPE_DEATH"] 		= 0,
	}
	
	-- Track last energy value
	self.lastPower = 0
	
	-- Updateto to new rev, data converters should go here
	self.db.profile.lastMinor = Avion2_Runes.minor
end


-- (Un)register (un)needed events
function Avion2_Runes:UpdateEventRegistration()
	-- Runes events
	if (self:IsEnabled() and next(self.db.profile.combinations.data)) then
		self:RegisterEvent("RUNE_POWER_UPDATE", RuneRelay)
		self:RegisterEvent("RUNE_TYPE_UPDATE", RuneRelay)
		self:RegisterEvent("UNIT_RUNIC_POWER", PowerRelay)
	end
	if (self:IsEnabled() and next(self.db.profile.power.data)) then
		self:RegisterEvent("UNIT_RUNIC_POWER", PowerRelay)
	end
	if (self:IsEnabled() and next(self.db.profile.runes.data)) then
		self:RegisterEvent("RUNE_POWER_UPDATE", RuneRelay)
		self:RegisterEvent("RUNE_TYPE_UPDATE", RuneRelay)
	end
	
	if (not self:IsEnabled() or not (next(self.db.profile.combinations.data) or next(self.db.profile.power.data) or next(self.db.profile.runes.data))) then
		self:UnregisterEvent("RUNE_POWER_UPDATE")
		self:UnregisterEvent("RUNE_TYPE_UPDATE")
		self:UnregisterEvent("UNIT_RUNIC_POWER")
	end
end


-- Update animation (convert animationFunc: string name to function pointer)
function Avion2_Runes:LoadAnimationTables()
	for _, info in pairs(self.db.profile.combinations.data) do
		Avion2Core:LoadAnimationTables(info.animationList)
	end
	
	for _, data in pairs(self.db.profile.power.data) do
		Avion2Core:LoadAnimationTables(data.animationList)
	end
	
	for _, data in pairs(self.db.profile.runes.data) do
		Avion2Core:LoadAnimationTables(data.animationList)
	end
end


-------------------------------------------------------------------------------
-- Addon - Runes - Options
-------------------------------------------------------------------------------
-- Fetch animationTable from animationID (as suplied by eg. Menu_Reuqest)
function Avion2_Runes:GetAnimationTable(animationID)
	success, _, arg1, arg2, arg3, arg4 = self:Deserialize(animationID)
	
	if (success) then
		if (arg1 == "Power") then
			return self.db.profile.power.data[arg2].animationList[arg3]
		elseif (arg1 == "Runes") then
			return self.db.profile.runes.data[arg2].animationList[arg3]
		elseif (arg1 == "Combination") then
			return self.db.profile.combinations.data[arg2].animationList[arg3]
		end
	else
		return false
	end
end


-- Build Optionstable
function Avion2_Runes:UpdateOptionTable(args)
	-- Update options when enabled and initialized
	if (self.db and self:IsEnabled()) then
		local serializedInfo
		
		-- Final options (POWER HERE)
		args[self.major.."_Power"] = {
			type = 'group',
			name = L["ThresholdsPower"],
			desc = L["ThresholdsPower_Desc"],
			args = {
				new = {
					order = 1,
					type = 'input',
					name = L["NewThreshold"],
					desc = L["NewThresholdPower_Desc"],
					usage = L["NewThresholdPower_Usage"],
					pattern = "(%d)",
					get = function()
						return ""
					end,
					set = function(info, val)
						if (not self.db.profile.power.data[tonumber(val)]) then
							self.db.profile.power.data[tonumber(val)] = {
								animationList 	= {},
								cancelAnimation	= 0,
							}
							
							self:UpdateEventRegistration()
							Avion2Core:UpdateModuleOptionTable(self)
						end
					end,
				},
				onlyOneActive = {
					order = 2,
					type = 'toggle',
					name = L["OnlyOneActive"],
					desc = L["OnlyOneActive_Desc"],
					get = function()
						return self.db.profile.power.onlyOneActive
					end,
					set = function()
						self.db.profile.power.onlyOneActive = not self.db.profile.onlyOneActive
					end,
				},
			},
		}		
		-- Options for each power threshold
		for index, data in pairs(self.db.profile.power.data) do
			args[self.major.."_Power"].args[string.format("Threshold_%d", index)] = {
				type = 'group',
				name = string.format(L["Threshold"], index),
				desc = string.format(L["ThresholdPower_Desc"], index),
				args = { 
					list = {
						order = 1,
						type = 'group',
						name = coreL["AnimationList"],
						desc = coreL["AnimationList_Desc"],
						args = {
							new = Avion2Core:GetDefaultNewAnimationOption(data.animationList),
						},
					},
					cancelAnimation = {
						order = 2,
						type = 'select',
						name = coreL["CancelAnimation"],
						desc = coreL["CancelAnimation_Desc"],
						values = {
							[0] = coreL["DontCancel"],
							[1] = coreL["FadeOut"],
							[2] = coreL["Cancel"],
						},
						get = function()
							return data.cancelAnimation
						end,
						set = function(info, val)
							data.cancelAnimation = val
						end,
					},
					delete = {
						order = 3,
						type = 'execute',
						name = L["DeleteThreshold"],
						desc = L["DeleteThreshold_Desc"],
						func = function()
							Avion2Core:StopAnimationList(string.format("Avion2_RunesThreshold_%d", index), data.animationList, false)
							self.db.profile.power.data[index] = nil
							
							self:UpdateEventRegistration()
							Avion2Core:UpdateModuleOptionTable(self)
						end,
					},
				},
			}
			-- Add options for each animation
			for animationListID, _ in pairs(data.animationList) do
				-- Combine all info to find this animation in module db in ONE string
				serializedInfo = self:Serialize(Avion2_Runes.major, "Power", index, animationListID)
				
				-- Update option table
				args[self.major.."_Power"].args[string.format("Threshold_%d", index)].args.list.args[string.format("Animation_%d", animationListID)] = Avion2Core:GetDefaultAnimationOption(data.animationList, animationListID, serializedInfo)
			end
		end	

		-- Final options (RUNES HERE)
		args[self.major.."_Runes"] = {
			type = 'group',
			name = L["ThresholdsRunes"],
			desc = L["ThresholdsRunes_Desc"],
			args = {
				new = {
					type = 'group',
					order = 1,
					name = L["NewRunes"],
					desc = L["NewRunes_Desc"],
					args = {
						runes = {
							order = 1,
							type = 'input',
							name = L["NewRune"],
							desc = L["NewRune_Desc"],
							usage = L["NewRune_Usage"],
							pattern = "(%d)",
							get = function()
								return tostring(newSettings.rune)
							end,
							set = function(info, val)
								newSettings.rune = tonumber(val)
							end,
						},
						runeType = {
							order = 2,
							type = 'select',
							name = L["NewRuneType"],
							desc = L["NewRuneType_Desc"],
							values = {
								["RUNETYPE_BLOOD"] 		= L["RUNETYPE_BLOOD"],
								["RUNETYPE_CHROMATIC"] 	= L["RUNETYPE_CHROMATIC"],
								["RUNETYPE_FROST"] 		= L["RUNETYPE_FROST"],
								["RUNETYPE_DEATH"] 		= L["RUNETYPE_DEATH"],
							},
							get = function()
								return newSettings.runeType
							end,
							set = function(info, val)
								newSettings.runeType = val
							end,
						},
						addMe = {
							order = 3,
							type = 'execute',
							name = L["AddMe"],
							desc = L["AddMe_Desc"],
							func = function(info)
								if (newSettings.rune >= 0 and newSettings.power >= 0) then
									-- Igor, did we find it??? Igor?
									local found = false
										
									-- Search it
									for _, info in pairs(self.db.profile.runes.data) do
										if (info.runes == newSettings.rune and info.runeType == newSettings.runeType) then
											found = true
											break
										end
									end
									
									-- Add if there isn't allready one
									if (not found) then
										-- Add
										table.insert(self.db.profile.runes.data, {runes = newSettings.rune, runeType = newSettings.runeType, cancelAnimation = 0, animationList = {}})
										
										-- Stop all running animation (due to resorting of animation table) NOTE: Only <need> to stop this modules own animaions, .. well.
										--!!!LibAvion:StopAllAnimations(false)
										
										-- Update registration of events and (oh shit) we need to recreate options table, since we resorted everything -.-
										self:UpdateEventRegistration()
										Avion2Core:UpdateModuleOptionTable(self)
									end
								end
							end
						},
					},
				},
				onlyOneActive = {
					order = 2,
					type = 'toggle',
					name = L["OnlyOneActive"],
					desc = L["OnlyOneActive_Desc"],
					get = function()
						return self.db.profile.runes.onlyOneActive
					end,
					set = function()
						self.db.profile.runes.onlyOneActive = not self.db.profile.runes.onlyOneActive
					end,
				},
			},
		}		
		-- Options for each rune
		for index, data in pairs(self.db.profile.runes.data) do
			args[self.major.."_Runes"].args[string.format("Threshold_%d", index)] = {
				type = 'group',
				name = string.format(L["Threshold"], index),
				desc = string.format(L["ThresholdRunes_Desc"], index),
				args = { 
					list = {
						order = 1,
						type = 'group',
						name = coreL["AnimationList"],
						desc = coreL["AnimationList_Desc"],
						args = {
							new = Avion2Core:GetDefaultNewAnimationOption(data.animationList),
						},
					},
					cancelAnimation = {
						order = 2,
						type = 'select',
						name = coreL["CancelAnimation"],
						desc = coreL["CancelAnimation_Desc"],
						values = {
							[0] = coreL["DontCancel"],
							[1] = coreL["FadeOut"],
							[2] = coreL["Cancel"],
						},
						get = function()
							return data.cancelAnimation
						end,
						set = function(info, val)
							data.cancelAnimation = val
						end,
					},
					delete = {
						order = 3,
						type = 'execute',
						name = L["DeleteThreshold"],
						desc = L["DeleteThreshold_Desc"],
						func = function()
							Avion2Core:StopAnimationList(string.format("Avion2_RunesThreshold_%d", index), data.animationList, false)
							self.db.profile.runes.data[index] = nil
							
							self:UpdateEventRegistration()
							Avion2Core:UpdateModuleOptionTable(self)
						end,
					},
				},
			}
			-- Add options for each animation
			for animationListID, _ in pairs(data.animationList) do
				-- Combine all info to find this animation in module db in ONE string
				serializedInfo = self:Serialize(Avion2_Runes.major, "Runes", index, animationListID)
				
				-- Update option table
				args[self.major.."_Runes"].args[string.format("Threshold_%d", index)].args.list.args[string.format("Animation_%d", animationListID)] = Avion2Core:GetDefaultAnimationOption(data.animationList, animationListID, serializedInfo)
			end
		end	
		
		-- Final options (COMBINATIONS HERE)
		args[self.major .. "_Combination"] = {
			type = 'group',
			name = L["Combination"],
			desc = L["Combination_Desc"],
			args = {
				new = {
					type = 'group',
					order = 1,
					name = L["NewCombination"],
					desc = L["NewCombination_Desc"],
					args = {
						power = {
							order = 1,
							type = 'input',
							name = L["NewPower"],
							desc = L["NewPower_Desc"],
							usage = L["NewPower_Usage"],
							pattern = "(%d)",
							get = function()
								return tostring(newSettings.energy)
							end,
							set = function(info, val)
								newSettings.energy = tonumber(val)
							end,
						},
						["RUNETYPE_BLOOD"] = {
							order = 2,
							type = 'input',
							name = L["RUNETYPE_BLOOD"],
							desc = L["NewRunes_Desc"],
							usage = L["NewRunes_Usage"],
							pattern = "(%d)",
							get = function()
								return tostring(newSettings.runes["RUNETYPE_BLOOD"])
							end,
							set = function(info, val)
								newSettings.runes["RUNETYPE_BLOOD"] = tonumber(val)
							end,
						},
						["RUNETYPE_CHROMATIC"] = {
							order = 3,
							type = 'input',
							name = L["RUNETYPE_CHROMATIC"],
							desc = L["NewRunes_Desc"],
							usage = L["NewRunes_Usage"],
							pattern = "(%d)",
							get = function()
								return tostring(newSettings.runes["RUNETYPE_CHROMATIC"])
							end,
							set = function(info, val)
								newSettings.runes["RUNETYPE_CHROMATIC"] = tonumber(val)
							end,
						},
						["RUNETYPE_FROST"] = {
							order = 4,
							type = 'input',
							name = L["RUNETYPE_FROST"],
							desc = L["NewRunes_Desc"],
							usage = L["NewRunes_Usage"],
							pattern = "(%d)",
							get = function()
								return tostring(newSettings.runes["RUNETYPE_FROST"])
							end,
							set = function(info, val)
								newSettings.runes["RUNETYPE_FROST"] = tonumber(val)
							end,
						},
						["RUNETYPE_DEATH"] = {
							order = 5,
							type = 'input',
							name = L["RUNETYPE_DEATH"],
							desc = L["NewRunes_Desc"],
							usage = L["NewRunes_Usage"],
							pattern = "(%d)",
							get = function()
								return tostring(newSettings.runes["RUNETYPE_DEATH"])
							end,
							set = function(info, val)
								newSettings.runes["RUNETYPE_DEATH"] = tonumber(val)
							end,
						},
						addMe = {
							order = 3,
							type = 'execute',
							name = L["AddMe"],
							desc = L["AddMe_Desc"],
							func = function(info)
								if (newSettings.runes["RUNETYPE_BLOOD"] >= 0 and newSettings.runes["RUNETYPE_CHROMATIC"] >= 0 and newSettings.runes["RUNETYPE_FROST"] >= 0 and newSettings.runes["RUNETYPE_DEATH"] >= 0 and newSettings.power >= 0) then
									-- Igor, did we find it??? Igor?
									local found = false
										
									-- Search it
									for _, info in pairs(self.db.profile.combinations.data) do
										if (info.runes["RUNETYPE_BLOOD"] == newSettings.runes["RUNETYPE_BLOOD"] and info.runes["RUNETYPE_CHROMATIC"] == newSettings.runes["RUNETYPE_CHROMATIC"] and info.runes["RUNETYPE_FROST"] == newSettings.runes["RUNETYPE_FROST"] and info.runes["RUNETYPE_DEATH"] == newSettings.runes["RUNETYPE_DEATH"] and info.power == newSettings.power) then
											found = true
											break
										end
									end
									
									-- Add if there isn't allready one
									if (not found) then
										-- Add
										table.insert(self.db.profile.combinations.data, {
											runes = {
												newSettings.runes["RUNETYPE_BLOOD"], 
												newSettings.runes["RUNETYPE_CHROMATIC"],
												newSettings.runes["RUNETYPE_FROST"],
												newSettings.runes["RUNETYPE_DEATH"],
											},
											power = newSettings.power, 
											cancelAnimation = 0, 
											animationList = {}
										})
										
										-- Stop all running animation (due to resorting of animation table) NOTE: Only <need> to stop this modules own animaions, .. well.
										--!!!LibAvion:StopAllAnimations(false)
										
										-- Update registration of events and (oh shit) we need to recreate options table, since we resorted everything -.-
										self:UpdateEventRegistration()
										Avion2Core:UpdateModuleOptionTable(self)
									end
								end
							end
						},
					},
				},
				onlyHighestRunes = {
					type = 'toggle',
					name = L["OnlyHighestRunes"],
					desc = L["OnlyHighestRunes_Desc"],
					get = function()
						return self.db.profile.combinations.onlyHighestRunes
					end,
					set = function()
						self.db.profile.combinations.onlyHighestRunes = not self.db.profile.combinations.onlyHighestRunes
					end,
				},
				onlyHighestPower = {
					type = 'toggle',
					name = L["OnlyHighestPower"],
					desc = L["OnlyHighestPower_Desc"],
					get = function()
						return self.db.profile.combinations.onlyHighestPower
					end,
					set = function()
						self.db.profile.combinations.onlyHighestPower = not self.db.profile.combinations.onlyHighestPower
					end,
				},
			},
		}		
		-- Insert menu for each trigger
		for key, info in pairs(self.db.profile.combinations.data) do 
			-- !!!
			-- args[self.major .. "_Combination"].args[string.format("Runes_%d-Power_%d", info.comboPoints, info.energy)]
			--args[self.major .. "_Combination"].args[key]
			-- !!!
			
			args[self.major .. "_Combination"].args[string.format("Runes_%d-Power_%d", info.comboPoints, info.energy)] = {
				type = 'group',
				name = string.format(L["ComboPointEnergyTitle"], info.comboPoints, info.energy),
				desc = L["ComboPointEnergyTitle_Desc"],
				args = { 
					list = {
						order = 1,
						type = 'group',
						name = coreL["AnimationList"],
						desc = coreL["AnimationList_Desc"],
						args = {
							new = Avion2Core:GetDefaultNewAnimationOption(info.animationList),
						},
					},
					cancelAnimation = {
						order = 2,
						type = 'select',
						name = coreL["CancelAnimation"],
						desc = coreL["CancelAnimation_Desc"],
						values = {
							[0] = coreL["DontCancel"],
							[1] = coreL["FadeOut"],
							[2] = coreL["Cancel"],
						},
						get = function()
							return info.cancelAnimation
						end,
						set = function(info, val)
							info.cancelAnimation = val
						end,
					},
					delete = {
						order = 3,
						type = 'execute',
						name = L["DeleteComboPointsEnergy"],
						desc = L["DeleteComboPointsEnergy_Desc"],
						func = function()
							Avion2Core:StopAnimationList(string.format("Avion2_ComboPointEnergy_%d-%d", info.comboPoints, info.energy), info.animationList, false)
							self.db.profile.combinations[key] = nil
							
							self:UpdateEventRegistration()
							Avion2Core:UpdateModuleOptionTable(self)
						end,
					},
				},
			}			
			-- Add options for each animation
			for animationListID, _ in pairs(info.animationList) do
				-- Combine all info to find this animation in module db in ONE string
				serializedInfo = self:Serialize(Avion2_Runes.major, "Combination", info.comboPoints, info.energy, animationListID)
				
				-- Update option table
				args[self.major .. "_Combination"].args[string.format("ComboPoint_%d-Energy_%d", info.comboPoints, info.energy)].args.list.args[string.format("Animation_%d", animationListID)] = Avion2Core:GetDefaultAnimationOption(info.animationList, animationListID, serializedInfo)
			end
		end

	-- Disabled, remove options
	else
		args[self.major .. "_Power"] 		= nil
		args[self.major .. "_Runes"] 		= nil
		args[self.major .. "_Combination"] 	= nil
	end
end


-------------------------------------------------------------------------------
-- Addon - Runes - Events
-------------------------------------------------------------------------------
local lastComboPoints, bestComboPointHit, highestEnergyThreshold, highestComboPointThreshold
local runningCache = {}


-- Relay function
ComboPointRelay = function(eventName, unitID)
	if (unitID == "player" and UnitPowerType("player") == 3) then
		ComboPointUpdate()
		ComboPointEnergyUpdate()
		
		-- Update last Runes
		Avion2_Energy.lastComboPoints 	= GetComboPoints()
	end
end
EnergyRelay = function(eventName, unitID)
	if (unitID == "player" and UnitPowerType("player") == 3) then
		EnergyUpdate()
		ComboPointEnergyUpdate()
	
		-- Update last energy
		Avion2_Energy.lastUnitPower 	= UnitPower("player")
	end
end


-- Energy changed
EnergyUpdate = function()
	-- Reset
	highestEnergyThreshold 	= -1
	
	-- For each threshold, trigger all animation since last update and cancel all old one
	for threshold, data in pairs(Avion2_Runes.db.profile.energy) do
		-- Check if Energy if greater than threshold
		if (Avion2_Runes.db.profile.onlyOneActiveEnergy or Avion2_Runes.db.profile.energy[threshold].cancelAnimation ~= 0 and UnitPower("player") < threshold) then -- and threshold < self.lastUnitPower
			Avion2Core:StopAnimationList(string.format("Avion2_EnergyThreshold_%d", index), Avion2_Runes.db.profile.energy[threshold].animationList, (not Avion2_Runes.db.profile.onlyOneActiveEnergy and Avion2_Runes.db.profile.energy[threshold].cancelAnimation == 1))
		end

		-- Check if Energy is inside interval [newEnergy - oldEnergy)
		if (Avion2_Runes.lastUnitPower < threshold and threshold <= UnitPower("player")) then
			-- If onlyOneActiveEnergy is set, keep track of loewst thresthold (if available) and don't fire an animation (yet)
			if (Avion2_Runes.db.profile.onlyOneActiveEnergy) then
				if (threshold > highestEnergyThreshold or highestEnergyThreshold == -1) then
					highestEnergyThreshold = threshold
				end
			else
				Avion2Core:TriggerAnimationList(string.format("Avion2_EnergyThreshold_%d", threshold), Avion2_Runes.db.profile.energy[threshold].animationList)
			end
		end
	end
	
	-- When onlyOneActiveEnergy is set, than highestEnergyThreshold is updated until highest threshold is found
	if (Avion2_Runes.db.profile.onlyOneActiveEnergy and Avion2_Runes.db.profile.energy[highestEnergyThreshold]) then
		Avion2Core:TriggerAnimationList(string.format("Avion2_EnergyThreshold_%d", threshold), Avion2_Runes.db.profile.energy[highestEnergyThreshold].animationList)
	end
end


-- Energy changed
ComboPointUpdate = function()
	-- Reset
	highestComboPointThreshold 	= -1
	
	-- For each threshold, trigger all animation since last update and cancel all old one
	for threshold, data in pairs(Avion2_Runes.db.profile.comboPoint) do
		-- Check if comboPoint if greater than threshold
		if (Avion2_Runes.db.profile.onlyOneActiveComboPoint or Avion2_Runes.db.profile.comboPoint[threshold].cancelAnimation ~= 0 and GetComboPoints() < threshold) then -- and threshold < self.lastUnitPower
			Avion2Core:StopAnimationList(string.format("Avion2_comboPointThreshold_%d", index), Avion2_Runes.db.profile.comboPoint[threshold].animationList, (not Avion2_Runes.db.profile.onlyOneActiveComboPoint and Avion2_Runes.db.profile.comboPoint[threshold].cancelAnimation == 1))
		end

		-- Check if comboPoint is inside interval [newcomboPoint - oldcomboPoint)
		if (Avion2_Runes.lastUnitPower < threshold and threshold <= GetComboPoints()) then
			-- If onlyOneActiveComboPoint is set, keep track of loewst thresthold (if available) and don't fire an animation (yet)
			if (Avion2_Runes.db.profile.onlyOneActiveComboPoint) then
				if (threshold > highestComboPointThreshold or highestComboPointThreshold == -1) then
					highestComboPointThreshold = threshold
				end
			else
				Avion2Core:TriggerAnimationList(string.format("Avion2_comboPointThreshold_%d", threshold), Avion2_Runes.db.profile.comboPoint[threshold].animationList)
			end
		end
	end
	
	-- When onlyOneActiveComboPoint is set, than highestComboPointThreshold is updated until highest threshold is found
	if (Avion2_Runes.db.profile.onlyOneActiveComboPoint and Avion2_Runes.db.profile.comboPoint[highestComboPointThreshold]) then
		Avion2Core:TriggerAnimationList(string.format("Avion2_comboPointThreshold_%d", threshold), Avion2_Runes.db.profile.comboPoint[highestComboPointThreshold].animationList)
	end
end


-- Combo point gained/spend or energy update
ComboPointEnergyUpdate = function()
	lastComboPoints 	= -1
	bestComboPointHit	= -1
	
	-- Iterate overl all Runes
	for _, info in ipairs(Avion2_Runes.db.profile.combinations) do
		-- No need to iterate if we exactCP and tableCP < currentCP
		if (Avion2_Runes.db.profile.onlyHighestComboPoints and bestComboPointHit ~= -1 and bestComboPointHit ~= lastComboPoints) then
			break
		end
	
		-- Test if Runes/Energy combination is found, that fits current values (and wasn't updated in the last push) ...
		if (GetComboPoints() 	>= info.comboPoints and info.comboPoints 	> Avion2_Energy.lastComboPoints				-- combopoints fits (not updated since last push)
		and UnitPower("player") >= info.energy 		and info.energy			> Avion2_Energy.lastUnitPower				-- energy fits (not updated since last push)
		and (not Avion2_Runes.db.profile.onlyHighestComboPoints or GetComboPoints() >= info.comboPoints)) then	-- 
			-- Need to know, when to cancel
			bestComboPointHit = info.comboPoints
			
			-- Only trigger animation if not onlyHighestEnergy or its the highestEnergy for this amount of cp
			if (not Avion2_Runes.db.profile.onlyHighestEnergy or lastComboPoints ~= GetComboPoints()) then
				Avion2Core:TriggerAnimationList(string.format("Avion2_ComboPointEnergy_%d-%d", info.comboPoints, info.energy), info.animationList)
			
				-- Track that we triggered this animation
				if (info.cancelAnimation ~= 0) then
					table.insert(runningCache, {comboPoints = info.comboPoints, energy = info.energy})
				end
			end
		end
		
		-- Need to keep track if iteration shows up lower Runes
		lastComboPoints = info.comboPoints
	end

	-- Cancel all alnimation that need to be canceled
	for key, info in pairs(runningCache) do
		-- Need to be canceled? (Because not active anymore)
		if (GetComboPoints() < info.ComboPoints or UnitPower("player") < info.energy) then
			Avion2Core:StopAnimationList(string.format("Avion2_ComboPointEnergy_%d-%d", info.ComboPoints, info.energy), info.animationList, (info.cancelAnimation == 1))
			
			-- Delete from track-list
			table.remove(runningCache, key)
		end
	end
end
